package com.tzld.longarticle.recommend.server.service.recommend;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoAuditStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoBadStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ContentPoolEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.PublishContentMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.VideoPoolAuditMapper;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.PublishSingleVideoSource;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.*;
import com.tzld.longarticle.recommend.server.model.vo.VideoPoolAuditListVO;
import com.tzld.longarticle.recommend.server.repository.longArticle.PublishSingleVideoSourceRepository;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service
@Slf4j
public class VideoPoolAuditService {

    @Autowired
    private VideoPoolAuditMapper videoPoolAuditMapper;
    @Autowired
    private PublishContentMapper publishContentMapper;
    @Autowired
    private PublishSingleVideoSourceRepository videoSourceRepository;

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Value("${cdnUrl:https://rescdn.piaoquantv.com/}")
    private String cdnUrl;

    @Value("${videoAudit.poolLevel.sort:}")
    private String poolLevelDesc;

    @ApolloJsonValue("${daily.video.audit.pool.count.config:{}}")
    private Map<String, Integer> dailyAuditPoolCount;

    public Page<VideoPoolAuditListVO> list(VideoPoolAuditListParam param) {
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        int count = videoPoolAuditMapper.articleVideoAuditListCount(param.getContentId(), param.getStatus(),
                param.getTitle(), param.getAuditAccount(), param.getFlowPoolLevel());
        List<PublishSingleVideoSource> list = videoPoolAuditMapper.articleVideoAuditList(param.getContentId(),
                param.getStatus(), param.getTitle(), param.getAuditAccount(), param.getFlowPoolLevel()
                , offset, param.getPageSize(), poolLevelDesc);
        List<VideoPoolAuditListVO> result = buildVideoPoolAuditListVO(list);
        Page<VideoPoolAuditListVO> page = new Page<>(param.getPageNum(), param.getPageSize());
        page.setTotalSize(count);
        page.setObjs(result);
        return page;
    }

    private List<VideoPoolAuditListVO> buildVideoPoolAuditListVO(List<PublishSingleVideoSource> list) {
        List<VideoPoolAuditListVO> result = new ArrayList<>();
        if (CollectionUtils.isEmpty(list)) {
            return result;
        }
        for (PublishSingleVideoSource item : list) {
            VideoPoolAuditListVO vo = new VideoPoolAuditListVO();
            vo.setContentId(item.getContentTraceId());
            vo.setTitle(item.getArticleTitle());
            vo.setStatus(item.getVideoPoolAuditStatus());
//            vo.setSynopsis(item.getSummaryText());
            vo.setFlowPoolLevel(ContentPoolEnum.from(item.getFlowPoolLevel()).getDescription());
            vo.setCrawlerTimestamp(item.getCrawlerTimestamp() * 1000);
            vo.setAuditAccount(item.getVideoPoolAuditAccount());
            if (Objects.nonNull(item.getVideoPoolAuditTimestamp())) {
                vo.setAuditTimestamp(item.getVideoPoolAuditTimestamp() * 1000);
            }
            VideoPoolAuditListVO.VideoItem videoItem = new VideoPoolAuditListVO.VideoItem();
            videoItem.setVideoId(item.getContentTraceId());
            videoItem.setTitle(item.getMiniProgramTitle());
            videoItem.setVideoUrl(cdnUrl + item.getVideoOssPath());
            videoItem.setCoverUrl(item.getCoverUrl());
            vo.setVideo(videoItem);
            result.add(vo);
        }
        return result;
    }

    public Page<VideoPoolAuditListVO> next(VideoPoolAuditListParam param) {
        if (Objects.nonNull(param.getFlowPoolLevel())) {
            param.setPageSize(1);
            return list(param);
        }
        Page<VideoPoolAuditListVO> result = new Page<>();
        // 根据配置判断当日是否审核完成 并 选择内容池返回
        PublishSingleVideoSource obj = null;
        List<String> excludePoolLevel = new ArrayList<>();
        Integer poolLevel = getAuditPoolLevel(excludePoolLevel);
        if (Objects.isNull(poolLevel)) {
            obj = videoPoolAuditMapper.articleVideoAuditNext(param.getContentId(),
                    param.getStatus(), param.getTitle(), param.getAuditAccount(), poolLevel);
        } else {
            do {
                obj = videoPoolAuditMapper.articleVideoAuditNext(param.getContentId(),
                        param.getStatus(), param.getTitle(), param.getAuditAccount(), poolLevel);
                if (Objects.nonNull(obj)) {
                    break;
                }
                excludePoolLevel.add(ContentPoolEnum.from(poolLevel).getContentPool());
                poolLevel = getAuditPoolLevel(excludePoolLevel);
            } while (Objects.nonNull(poolLevel));
        }
        if (Objects.isNull(obj)) {
            return result;
        }
        List<VideoPoolAuditListVO> list = buildVideoPoolAuditListVO(Collections.singletonList(obj));
        result.setObjs(list);
        return result;
    }

    private Integer getAuditPoolLevel(List<String> excludePoolLevel) {
        if (MapUtils.isEmpty(dailyAuditPoolCount)) {
            return null;
        }
        String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
        Set<String> keySet = dailyAuditPoolCount.keySet();
        keySet = keySet.stream().sorted().collect(Collectors.toCollection(LinkedHashSet::new));
        for (String poolLevel : keySet) {
            if (excludePoolLevel.contains(poolLevel)) {
                continue;
            }
            int target = dailyAuditPoolCount.get(poolLevel);
            String key = "video_audit_count_" + dateStr + "_" + poolLevel;
            int totalCount = Integer.parseInt(Optional.ofNullable(redisTemplate.opsForValue().get(key)).orElse("0"));
            if (target > totalCount) {
                ContentPoolEnum poolEnum = ContentPoolEnum.from(poolLevel);
                return poolEnum.getValue();
            }
        }
        return null;
    }

    public void auditArticle(ArticleAuditParam param) {
        PublishSingleVideoSource videoAudit = videoSourceRepository.getByContentTraceId(param.getContentId());
        Long now = System.currentTimeMillis();
        if (param.getStatus().equals(ArticleVideoAuditStatusEnum.REJECT.getCode())) {
            videoAudit.setBadStatus(ArticleVideoBadStatusEnum.TITLE_AIGC_REJECT_STATUS.getCode());
        } else {
            videoAudit.setBadStatus(ArticleVideoBadStatusEnum.TITLE_DEFAULT_STATUS.getCode());
        }
        videoAudit.setVideoPoolAuditStatus(param.getStatus());
        videoAudit.setVideoPoolAuditAccount(param.getAuditAccount());
        videoAudit.setVideoPoolAuditTimestamp(now);
        videoSourceRepository.save(videoAudit);
        if (param.getStatus() == 2) {
            // 审核不通过
            auditArticleReject(videoAudit);
        } else {
            // 审核通过
            auditArticlePass(videoAudit);
        }
        // 当日审核数+1
        addAuditCount(ContentPoolEnum.from(videoAudit.getFlowPoolLevel()).getContentPool());
    }

    private void addAuditCount(String poolLevel) {
        String dateStr = DateUtils.getCurrentDateStr("yyyyMMdd");
        String key = "video_audit_count_" + dateStr + "_" + poolLevel;
        int totalCount = Integer.parseInt(Optional.ofNullable(redisTemplate.opsForValue().get(key)).orElse("0"));
        redisTemplate.opsForValue().set(key, String.valueOf(totalCount + 1), 24, TimeUnit.HOURS);
    }

    private void auditArticlePass(PublishSingleVideoSource videoAudit) {
        // publish_content
        publishContentMapper.updatePublishContentStatus(videoAudit.getContentTraceId(),
                PublishContentStatusEnum.waiting_publish.getVal(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal());
    }

    private void auditArticleReject(PublishSingleVideoSource videoAudit) {
        // publish_content
        publishContentMapper.updatePublishContentStatus(videoAudit.getContentTraceId(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal(),
                PublishContentStatusEnum.waiting_publish.getVal());
    }

    public List<Object> getFilterValue(AuditFilterParam param) {
        String itemName = param.getItemName();
        if ("contentId".equals(itemName) || "title".equals(itemName) || "auditAccount".equals(itemName)) {
            return videoPoolAuditMapper.searchFilterValueByItemName(itemName, param.getSearchKeyword())
                    .stream().collect(Collectors.toList());
        } else {
            return new ArrayList<>();
        }
    }

    public void updateArticleTitle(ArticleTitleUpdateParam param) {
        long now = System.currentTimeMillis();
        videoPoolAuditMapper.updateArticleTitle(param, now);
    }

    public void updateVideoTitle(VideoTitleUpdateParam param) {
        long now = System.currentTimeMillis();
        videoPoolAuditMapper.updateVideoTitle(param, now);
    }

    @XxlJob("shuffleVideoPoolAuditGroup")
    public ReturnT<String> shuffleVideoPoolAuditGroup(String param) {
        List<String> auditUser = Arrays.asList("a", "b", "c", "d", "e", "f", "g", "h", "i", "j");
        List<PublishSingleVideoSource> contentIds = videoSourceRepository.getByVideoPoolAuditStatus(ArticleVideoAuditStatusEnum.WAITING.getCode());
        for (int i = 0; i < contentIds.size(); i++) {
            int per = i % auditUser.size();
            PublishSingleVideoSource audit = contentIds.get(i);
            audit.setVideoPoolAuditAccount(auditUser.get(per));
            videoSourceRepository.save(audit);
        }
        return ReturnT.SUCCESS;
    }
}
