package com.tzld.longarticle.recommend.server.web.recommend;

import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.*;
import com.tzld.longarticle.recommend.server.model.vo.ArticleVideoAuditListVO;
import com.tzld.longarticle.recommend.server.service.recommend.ArticleVideoAuditService;
import com.tzld.longarticle.recommend.server.util.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/articleVideoAudit")
@Slf4j
public class ArticleVideoAuditController {

    @Autowired
    ArticleVideoAuditService service;

    @PostMapping("/list")
    public CommonResponse<Page<ArticleVideoAuditListVO>> articleVideoAuditList(@RequestBody ArticleVideoAuditListParam param) {
        return CommonResponse.success(service.list(param));
    }

    @PostMapping("/next")
    public CommonResponse<Page<ArticleVideoAuditListVO>> articleVideoAuditNext(@RequestBody ArticleVideoAuditListParam param) {
        return CommonResponse.success(service.next(param));
    }

    @PostMapping("/auditArticle")
    public CommonResponse<Void> auditArticle(@RequestBody ArticleAuditParam param) {
        service.auditArticle(param);
        return CommonResponse.success();
    }

    @PostMapping("/auditVideo")
    public CommonResponse<Void> auditVideo(@RequestBody VideoAuditParam param) {
        service.auditVideo(param);
        return CommonResponse.success();
    }

    @PostMapping("/getFilterValue")
    public CommonResponse<List<Object>> getFilterValue(@RequestBody AuditFilterParam param) {
        return CommonResponse.success(service.getFilterValue(param));
    }


    @GetMapping("/rematchVideoAudit")
    public CommonResponse<Void> rematchVideoAudit(String contentId) {
        service.rematchVideoAudit(contentId);
        return CommonResponse.success();
    }


    @PostMapping("/updateArticleTitle")
    public CommonResponse<Void> updateArticleTitle(@RequestBody ArticleTitleUpdateParam param) {
        service.updateArticleTitle(param);
        return CommonResponse.success();
    }


    @PostMapping("/updateVideoTitle")
    public CommonResponse<Void> updateVideoTitle(@RequestBody VideoTitleUpdateParam param) {
        service.updateVideoTitle(param);
        return CommonResponse.success();
    }


    @GetMapping("/shuffleAuditGroup")
    public CommonResponse<Void> shuffleAuditGroup() {
        service.shuffleAuditGroup(null);
        return CommonResponse.success();
    }

    @PostMapping("/addAudit")
    public CommonResponse<Void> addAudit(@RequestBody ArticleVideoAuditAddAuditParam param) {
        service.addAudit(param);
        return CommonResponse.success();
    }

}
